from django.conf import settings

MB_DRF_EXTENSIONS_CONFIG = getattr(settings, 'MB_DRF_EXTENSIONS', {})
MB_AUTHENTICATIONS = MB_DRF_EXTENSIONS_CONFIG.get('AUTHENTICATIONS', {})
MB_EXCEPTIONS = MB_DRF_EXTENSIONS_CONFIG.get('EXCEPTIONS', {})
MB_VIEWS = MB_DRF_EXTENSIONS_CONFIG.get('VIEWS', {})
